import 'package:flutter/foundation.dart';

import 'package:firebase_admob/firebase_admob.dart';
import 'package:static_sharedpreferences/static_sharedpreferences.dart';

import '../easy_ads.dart';
import 'easy_rewarded_video_ad.dart';

/// ## Description
///
/// * You `have to initialize` [EasyAds.initialize], otherwise this class will not work.
///
///
/// * You `have to initialize` this class with [HealthyEasyRewardedVideoAd.initialize]
///   otherwise it will not work.
/// 
/// 
/// * This class helps you not to flood your users with ads.
/// 
///
/// ## Example
///
/// {@tool sample}
/// ```dart
/// void main(){
///   WidgetsFlutterBinding.ensureInitialized();
///
///   EasyAds.initialize();
///   HealthyEasyRewardedVideoAd.initialize(
///     easyRewardedVideoAd: EasyRewardedVideoAd(),
///     blockAdsUntil: 5,
///   );
///
///   runApp(MyApp());
/// }
///
/// class MyApp extends StatelessWidget {
///   @override
///   Widget build(BuildContext context) {
///     return MaterialApp(
///       title: 'EasyAds Demo',
///       theme: ThemeData(
///         primarySwatch: Colors.blue,
///       ),
///       home: MyAdWidget(),
///     );
///   }
/// }
///
/// class MyAdWidget extends StatefulWidget {
///   @override
///   _MyAdWidgetState createState() => _MyAdWidgetState();
/// }
///
/// class _MyAdWidgetState extends State<MyAdWidget> {
///
///   @override
///   Widget build(BuildContext context) {
///     return Scaffold(
///       body: Container(
///         child: Center(
///           child: RaisedButton(
///             // Because we set blockAdsUntil = 5 in the
///             // HealthyEasyRewardedVideoAd.initialize
///             // method, the ad will not be shown until the
///             // user has clicked the button 5 times
///             onPressed: HealthyEasyRewardedVideoAd.show,
///           ),
///         ),
///       ),
///     );
///   }
/// }
/// ```
/// {@end-tool}
abstract class HealthyEasyRewardedVideoAd {
  static EasyRewardedVideoAd get easyRewardedVideoAd => _videoAd;

  static EasyRewardedVideoAd _videoAd;

  static int _count;

  /// ## Description
  ///
  /// * Can not be changed after initialization
  ///
  ///
  /// * Amount of times a certain event has to
  ///   be executed until the `EasyRewardedVideoAd` will be shown
  ///
  ///
  /// * This will be initialized with `20`
  static int get blockAdsUntil => _blockAdsUntil;

  static int _blockAdsUntil;

  static int _blockAdsUntilCount;

  static void initialize({
    @required EasyRewardedVideoAd easyRewardedVideoAd,
    int blockAdsUntil = 20,
  }) {
    _videoAd = easyRewardedVideoAd;

    _count = 0;

    _blockAdsUntil = blockAdsUntil;
    _blockAdsUntilCount =
        SP.getInt(EasyAds.SP_KEY + "_blockAdsUntilCount") ?? 0;
    ;
  }

  static Future<bool> load({MobileAdTargetingInfo targetingInfo}) =>
      (targetingInfo != null)
          ? _videoAd.load(mobileAdtargetingInfo: targetingInfo)
          : _videoAd.load();

  static Future<bool> show() {
    if (_blockAdsUntil <= _blockAdsUntilCount) {
      return _videoAd.show();
    } else {
      _blockAdsUntilCount++;
      SP.setInt(EasyAds.SP_KEY + "_blockAdsUntilCount", _blockAdsUntilCount);
      return null;
    }
  }

  /// ## Description
  ///
  /// * The Example assumes, that you have already initialized the
  ///   HealthyEasyRewardedVideoAd.
  /// 
  /// 
  /// ## Parameters
  /// 
  /// * `onValue`:
  /// 
  ///   Assuming `onValue = 5`, then every fifth time this function gets
  ///   executed an ad will be shown.
  /// 
  /// 
  /// * `onShowAd`: 
  /// 
  ///   This function gets executed right before the [EasyRewardedVideoAd.show] function.
  /// 
  /// 
  /// * `onNotShowAd`:
  /// 
  ///   In case the expression that evaluates if an ad will be shown,
  ///   using onValue and a private static counter variable equals false,
  ///   then this function gets executed.
  /// 
  /// 
  /// * `showAdOnValue`:
  /// 
  ///   In case you do not want this function to automatically show an ad
  ///   if the expression that evaluates if an ad will be shown equals true,
  ///   then set this to `false`.
  /// 
  /// 
  /// ## Example
  /// 
  /// {@tool sample}
  /// ```dart
  /// class MyAdWidget extends StatelessWidget {
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     return Scaffold(
  ///       body: Container(
  ///         child: Center(
  ///           child: RaisedButton(
  ///             onPressed: () {
  ///               // Because we set onValue: 3,
  ///               // every third button press will show an ad,
  ///               // but before the ad is shown onShowAd
  ///               // will get executed if it is non-null.
  ///               //
  ///               // In case the user pressed the button for the first or
  ///               // second time, onNotShowAd will get executed
  ///               // if non-null.
  ///               HealthyEasyRewardedVideoAd.showOnCount(
  ///                 onValue: 3,
  ///                 onShowAd: () {
  ///                   print("now showing ad");
  ///                 },
  ///                 onNotShowAd: () {
  ///                   print("u r lucky, no ad");
  ///                 },
  ///               );
  ///             },
  ///           ),
  ///         ),
  ///       ),
  ///     );
  ///   }
  /// }
  /// ```
  /// {@end-tool}
  static void showOnCount({
    int onValue = 5,
    Function onShowAd,
    Function onNotShowAd,
    bool showAdOnValue = true,
  }) {
    if (_count == onValue) {
      _count = 0;
      if (onShowAd != null) onShowAd();
      if (showAdOnValue) _videoAd.show();
    } else {
      _count++;
      if (onNotShowAd != null) onNotShowAd();
    }
  }
}
